#
# Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
#
# You should have find a copy of NFSChina License. If not, please contact us
# by email <os_support@nfschina.com>
#

import threading
import queue
import time
import os
from libfoundai.model_adapter.model_manager import AIModelsManager
from libfoundai.ai_ability_manager import AIAbilityManager
from libfoundai.utils.ai_framwork_config import Config



# 任务会话管理类
class TaskSessionManager:
    def __init__(self):
        self.sessions = {}
        self.session_id_counter = 0

    def create_session(self, task, model):
        session_id = self.session_id_counter
        self.session_id_counter += 1
        self.sessions[session_id] = {
            "task": task,
            "model": model,
            "status": "pending",
            "result": None
        }
        return session_id

    def update_session_status(self, session_id, status):
        if session_id in self.sessions:
            self.sessions[session_id]["status"] = status

    def get_session_result(self, session_id):
        if session_id in self.sessions:
            return self.sessions[session_id]["result"]
        return None


# 任务调度
class TaskScheduler:
    def __init__(self):
        self.task_queue = queue.Queue()

    def schedule_task(self, task):
        self.task_queue.put(task)

    def run(self):
        while True:
            if not self.task_queue.empty():
                task = self.task_queue.get()
                available_model = self.find_available_model()
                if available_model:
                    session_id = task_session_manager.create_session(task, available_model.model_name)
                    available_model.is_busy = True
                    task_session_manager.update_session_status(session_id, "running")
                    result = available_model.execute_task(task)
                    task_session_manager.update_session_status(session_id, "completed")
                    task_session_manager.sessions[session_id]["result"] = result
                else:
                    print("当前没有可用模型，任务暂无法执行")
            time.sleep(1)




class FoundAIManager:
    def __init__(self):
        self.ai_ability_manager = AIAbilityManager()
        self.ability_match_model = Config.get_ability_match_model()
        if self.ability_match_model:
            for ability, matchmodels in self.ability_match_model.items():
                for model in matchmodels:
                    self.ai_ability_manager.register_ability_model(ability, model)
    

        self.ai_model_manager = AIModelsManager()

        #TODO：后续考虑同步、异步接口、以及任务会话多线程处理
        '''
        self.task_session_manager = TaskSessionManager()

        self.model_scheduler = TaskScheduler()

        # 启动模型调度线程
        scheduler_thread = threading.Thread(target=self.model_scheduler.run)
        scheduler_thread.start()
        '''

    def create_task(self, ability, input_param):
        #session_id 暂时未用到，后续异步接口考虑使用，先预留
        session_id = 0
        res = ""
        model_instance = None
        try:
            model_list = self.ai_ability_manager.get_model_for_ability(ability)
            for model_name in model_list:
                if model_name:
                    #找到第一个可用的模型实例
                    model_instance = self.ai_model_manager.get_available_model(model_name)
                    if model_instance:
                        break
            
            if not model_instance:
                    res = "failed, no available AI model"
                    return False, session_id, res
            
            #根据模型的注册表，找对应的模型接口
            if ability not in model_instance.interface_registry:
                res = f"failed, {model_name} no match interface"
                return False, session_id, res

            flag, res = model_instance.call_interface(ability, input_param)
            return flag, session_id, res
        
        except Exception as e:
            res = f"generate_response exception :{e}"
            return False, session_id, res

    def req_model(self, models):
        return self.ai_model_manager.req_model(models)
    
    def chat_text(self, prompt):
        ability = "chat_text"
        return self.create_task(ability,prompt)

        '''
        session_id = 0
        res = ""
        model_instance = None
        try:
            model_name = self.ai_ability_manager.get_model_for_ability(ability)
            if model_name:
                #找到可用的模型实例
                model_instance = self.ai_model_manager.get_available_model(model_name)
                if not model_instance:
                    res = "failed, no available AI model"
                    return False, session_id, res

            else:
                res = "failed, no match model"
                return False, session_id, res
            
            #根据模型的注册表，找对应的模型接口
            if ability not in model_instance.interface_registry:
                res = f"failed, {model_name} no match interface"
                return False, session_id, res

            flag, res = model_instance.call_interface(ability, prompt)
            return flag, session_id, res
        
        except Exception as e:
            res = f"generate_response exception :{e}"
            return False, session_id, res
        '''


    def get_content_ocr(self, image_path):
        if not os.path.isfile(image_path):
            #print("image does not exist.")
            return False, 0, "image does not exist"
        ability = "get_content_ocr"
        return self.create_task(ability, image_path)
        
